unit hiTAR;

interface

uses Windows, KOL, Share, Debug;

type
  TCheckMode = (cmAuto, cmUser);

type
  PInfo = ^TInfo;
  TInfo = record
    size: integer;
    plugin, name, version, description,
    author, copyright, homepage: PChar;
  end;
  
  PExec = ^TExec;
  TExec = record
    size: integer;
    run: integer;
    con_text: PChar;
    con_sel_start, con_sel_length: integer;
  end;
  
type
  THiTAR = class(TDebug)
  private
    FParamsA: PArray;
    FParams, FAliases: PStrList;     
    FExec: ^TExec;
    FRunString: string;
    FIndex: integer;

    function _PGet(var Item: TData; var Val: TData): boolean;
    function _PCount: integer;

    procedure SetAliases(const Value: string);
    
    procedure Init;
  public
    ParentHandle: HWND;

    _prop_Name,
    _prop_Version,
    _prop_Description,
    _prop_Author,
    _prop_Copyright,
    _prop_Homepage: PChar;
    
    _prop_CheckMode: TCheckMode;
    _prop_Default: integer;
    
    _event_onExec: THI_Event;
    _event_onLoadPlugin: THI_Event;
    _event_onUnloadPlugin: THI_Event;

    constructor Create; reintroduce;
    destructor Destroy; override;

    procedure _work_doResult(var _Data: TData; Index: word);
    
    procedure _work_doSetConsoleText(var _Data: TData; Index: word);
    procedure _work_doSetAndSelectConsoleText(var _Data: TData; Index: word);
    procedure _work_doSetConsoleSelectionStart(var _Data: TData; Index: word);
    procedure _work_doSetConsoleSelectionLength(var _Data: TData; Index: word);
    procedure _work_doResetConsoleText(var _Data: TData; Index: word);
    
    procedure _work_doShiftParams(var _Data: TData; Index: word);
    procedure _work_doAddAlias(var _Data: TData; Index: word);
    
    procedure _var_Handle(var _Data: TData; Index: word);
    procedure _var_Params(var _Data: TData; Index: word);
    procedure _var_RunString(var _Data: TData; Index: word);
    procedure _var_Index(var _Data: TData; Index: word);
    
    //       ,    
    procedure _var_FirstParam(var _Data: TData; Index: word);
    
    property _prop_Aliases: string write SetAliases;
    property Aliases: PStrList read FAliases;
    property Params: PStrList read FParams;
    property Alias: integer read FIndex;
  end;
  
function IndexOf_NoCase(List: PStrList; const Value: string): integer;
function _Trim(const s: string): string;
  
var
  ProjectClass: THiTAR;
  
function GetInfo: TInfo; cdecl;
function GetCount: integer; cdecl;
function GetString(num: integer): PChar; cdecl;
function RunString(str: PChar): TExec; cdecl;

exports
  GetInfo,
  GetCount,
  GetString,
  RunString;

implementation

function GetInfo;
begin
  with ProjectClass, Result do
    begin
      Size := SizeOf(TInfo);
      plugin := 'Hello! I am the TypeAndRun plugin.';
      name := _prop_name;
      version := _prop_version;
      description := _prop_description;
      author := _prop_author;
      copyright := _prop_Copyright;
      homepage := _prop_homepage;
    end;
end; { of function GetInfo
-----------------------------------------------------------}

function GetCount;
begin
  Result := ProjectClass.FAliases.Count;
end; { of function GetCount
-----------------------------------------------------------}

function GetString;
begin
  Result := ProjectClass.FAliases.ItemPtrs[num];
end; { of function GetString
-----------------------------------------------------------}

function IndexOf_NoCase(List: PStrList; const Value: string): integer;
begin
  for Result := 0 to List.Count - 1 do
    if StrComp( PChar( KOL.LowerCase(Value) ), PChar( KOL.LowerCase(List.Items[Result]) ) ) = 0 then Exit;
  Result := -1;
end;

function _Trim(const s: string): string;
var
  i: integer;
begin
  Result := s;
  if Length(Result) > 0 then
    begin
      i := 1;
      while (i <= Length(Result)) and (Result[i] < #20) do
        inc(i);
      delete(Result, 1, i - 1);
      i := Length(Result);
      while (i > 0) and (Result[i] < #20) do
        dec(i);
      delete(Result, i + 1, Length(Result) - i);
    end;
end;

procedure GetParams(List: PStrList; s: string);
const _p: array[0..1] of char = (' ', '"');
var
  p: integer;
  f: byte;
begin
  List.Clear;
  s := Trim(s);
  while Length(s) > 0 do
    begin
      f := byte(s[1] = '"');
      p := PosEx(_p[f], s, 2);
      if p = 0 then p := Length(s) + 1;
      List.Add(_Trim(Copy(s, f * 1 + 1, p - f * 1 - 1)));
      Delete(s, 1, p);
      s := _Trim(s);
    end;
end;

function RunString;
begin
  FillChar(Result, SizeOf(TExec), 0);
  with ProjectClass, Result do
    begin
      Size := SizeOf(TExec);
      GetParams(FParams, str);
      FIndex := IndexOf_NoCase(FAliases, FParams.Items[0]);
      if (_Prop_CheckMode <> cmAuto) or (FIndex <> -1) then
        begin
          FRunString := str;
          FExec := @Result;
          if _Prop_CheckMode = cmAuto then
            begin
              Run := _prop_Default;
               _hi_onEvent(_event_onExec, FIndex);
            end
          else
            begin
              Run := 0;
              _hi_onEvent(_event_onExec, FRunString);
            end;
          FExec := nil;
          FRunString := '';
        end
      else
        Run := 0;
    end;
end; { of function RunString
-----------------------------------------------------------}

constructor THiTAR.Create;
begin
  inherited;
  FParams := NewStrList;
  FAliases := NewStrList;
  FParamsA := CreateArray(nil, _PGet, _PCount, nil);
  InitAdd(Init);
end;

destructor THiTAR.Destroy;
begin
  FParams.Free;
  FAliases.Free;
  inherited;
end;

procedure THiTAR.Init;
begin
  _hi_onEvent(_event_onLoadPlugin);
end;

procedure THiTAR.SetAliases;
begin
  FAliases.Text := Value;
end;

function THiTAR._PGet;
var
  idx: integer;
begin
  idx := ToIntIndex(Item);
  if (idx >= 0) and (idx < FParams.Count) then
    begin
      Result := true;
      dtString(Val, FParams.Items[idx]);
    end
  else Result := false;
end;

function THiTAR._PCount;
begin
  Result := FParams.Count;
end;

//------------------------------------------------------------

procedure THiTAR._work_doResult;
var
  x: integer;
begin
  x := ToInteger(_Data);
  if x <> FExec.Run then
    if x = 0 then FExec.Run := 0
    else if FExec.Run = 0 then FExec.Run := 1;
end;

procedure THiTAR._work_doSetConsoleText;
begin
  FExec.con_text := PChar(ToString(_data));
  FExec.Run := 2;
end;

procedure THiTAR._work_doSetAndSelectConsoleText;
begin
  FExec.con_text := PChar(ToString(_data));
  FExec.Run := 2;
  FExec.con_sel_length := lstrlen(FExec.con_text);
end;

procedure THiTAR._work_doSetConsoleSelectionStart;
begin
  FExec.con_sel_start := ToInteger(_data);
end;

procedure THiTAR._work_doSetConsoleSelectionLength;
begin
  FExec.con_sel_length := ToInteger(_data);
end;

procedure THiTAR._work_doResetConsoleText;
begin
  with FExec^ do
    begin
      if Run = 2 then Run := 1;
      con_sel_start := 0;
      con_sel_length := 0;
      con_text := '';
    end;
end;

procedure THiTAR._work_doShiftParams;
begin
  FParams.Delete(0);
end;

procedure THiTAR._work_doAddAlias;
begin
  FAliases.Add(ToString(_Data));
end;

procedure THiTAR._var_Handle;
begin
  dtInteger(_data, ParentHandle);
end;

procedure THiTAR._var_Params;
begin
  dtArray(_Data, FParamsA);
end;

procedure THiTAR._var_RunString;
begin
  dtString(_Data, FRunString);
end;

procedure THiTAR._var_Index;
begin
  dtInteger(_Data, FIndex);
end;

procedure THiTAR._var_FirstParam;
begin
  dtString(_Data, FParams.Items[1]);
end;

end.

