library make_TAR;

uses
  Windows,
  KOL,
  CGTShare in '..\..\CGTShare.pas';

type
  PCGrec = ^TCGrec;
  TCGrec = record
    MainForm: string;
    Vars, Units, IBody, Dead: PStrList;
  end;

function GetAppName: string;
begin
  SetLength(result, 1024);
  SetLength(result, GetModuleFileName(HInstance, PChar(@result[ 1 ]), 1024));
end;

function buildGetParamsProc(var params: TBuildParams): integer; cdecl;
begin
  params.flags := CGMP_COMPRESSED or CGMP_RUN or CGMP_RUN_DEBUG;
  Result := CG_SUCCESS;
end;

function GetClassName(const ElementName: string): string;
var
  i, Len: integer;
begin
  Len := Length(ElementName);
  for i := Len downto 1 do
    if ElementName[i] = '_' then
      begin
        Dec(PInteger(@i)^);
        break;
      end;
  if i <= 1 then
    i := Len;
  Result := 'THi' + Copy(ElementName, 1, i);
end;

function buildMakePrj(const params: TBuildMakePrjRec): integer; cdecl;
var
  R: PStrList;
  i: integer;
  P: PCGrec;
begin
  p := params.result;
  R := NewStrList;
  R.Add('library TARPlugin;');
  R.Add('');
  R.Add('uses');
  for i := 0 to p.Units.Count - 1 do
    R.Add('  ' + p.Units.Items[i] + ',');
  R.Add('  KOL, Share;');
  R.Add('');
  R.Add('{$I share.inc}');
  R.Add('{$R allres.res}');
  R.Add('');
  R.Add('type');
  R.Add('  TInitClass = class');
  R.Add('  public');
  R.AddStrings(p.Vars);
  R.Add('    constructor Create(ParentHandle: HWND); reintroduce;');
  R.Add('    destructor Destroy; override;');
  R.Add('  end;');
  R.Add('');
  R.Add('var');
  R.Add('  InitClass: TInitClass;');
  R.Add('');
  R.Add('constructor TInitClass.Create;');
  R.Add('begin');
  R.Add('  inherited Create;');
  R.Add('  InitClass := self;');
  R.AddStrings(p.IBody);
  R.Add('  ProjectClass := ' + p.MainForm + ';');
  R.Add('  ProjectClass.ParentHandle := ParentHandle;');
  R.Add('  EventOn;');
  R.Add('  InitDo;');
  R.Add('end;');
  R.Add('');
  R.Add('destructor TInitClass.Destroy;');
  R.Add('begin');
  R.Add('  _hi_onEvent(ProjectClass._event_onUnloadPlugin);');
  R.Add('  EventOff;');
  R.AddStrings(p.Dead);
  R.Add('  inherited;');
  R.Add('end;');
  R.Add('');
  R.Add('procedure Load(WND: HWND); cdecl;');
  R.Add('begin');
  R.Add('  TInitClass.Create(WND);');
  R.Add('end; { of procedure Load');
  R.Add('-----------------------------------------------------------}');
  R.Add('');
  R.Add('procedure Unload; cdecl;');
  R.Add('begin');
  R.Add('  InitClass.Free;');
  R.Add('end; { of procedure Unload');
  R.Add('-----------------------------------------------------------}');
  R.Add('');
  R.Add('exports');
  R.Add('  Load,');
  R.Add('  Unload;');
  R.Add('');
  R.Add('end.');

  R.SaveToFile(params.prjFilename);
  R.free;

  Result := CG_SUCCESS;
end;

function buildCompliteProc(const params: TBuildCompliteRec): integer; cdecl;
var
  src: string;
begin
  src := ExtractFilePath(params.prjFilename) + ExtractFileNameWOext(params.prjFilename) + '.dll';
  MoveFileEx(PChar(src), PChar(params.appFilename), MOVEFILE_COPY_ALLOWED or MOVEFILE_REPLACE_EXISTING);
  Result := CG_SUCCESS;
end;

function buildRunProc(var params: TBuildRunRec): integer; cdecl;
var
  ipath, iparams: string;
  procedure ReadIni;
  begin
    with OpenIniFile(ChangeFileExt(GetAppName, '.ini'))^ do
      begin
        Section := 'make';
        ipath := ValueString('Path', '');
        iparams := ValueString('Params', '');
        Free;
        if (iparams > '') then iparams := ' ' + iparams;
      end;

  end;
var
  si: TStartupInfo;
  p: TProcessInformation;
  res: cardinal;
begin
  FillChar(Si, SizeOf(Si), 0);
  with Si do
    begin
      cb := SizeOf(Si);
      dwFlags := STARTF_USESHOWWINDOW;
      wShowWindow := 4;
    end;
  ReadIni;
  CreateProcess(nil, PChar('"' + ipath + '" ' + iparams), nil, nil, false, CREATE_DEFAULT_ERROR_MODE, nil, nil, si, p);
  res := WAIT_TIMEOUT;
  Params.data := pointer(p.hProcess);
  while (res = WAIT_TIMEOUT) do
    res := WaitForSingleObject(p.hProcess, 10);

  Result := CG_SUCCESS;
end;

function buildStopProc(var params: TBuildRunRec): integer; cdecl;
begin
  TerminateProcess(cardinal(Params.data), 0);
  Result := CG_SUCCESS;
end;

exports
  buildGetParamsProc,
  buildMakePrj,
  buildCompliteProc,
  buildRunProc,
  buildStopProc;

begin


end.

